/*! Tripetto Collector 1.7.1-18 - Copyright (C) 2019 Tripetto B.V. - All Rights Reserved */

/// <reference path="marblecore.matrix.d.ts" />
/// <reference path="marblecore.serializer.d.ts" />
/// <reference path="marblecore.hookup.d.ts" />

declare module "@tripetto/slots" {
    /** Explicit exports */
    export { Slots } from "@tripetto/slots/lib/slots";
    export { Slot } from "@tripetto/slots/lib/slot";
    export { ISlot } from "@tripetto/slots/lib/interface";
    export { ISlotProperties } from "@tripetto/slots/lib/properties";
    export { ISlotType } from "@tripetto/slots/lib/type";
    /** Types */
    export { Kinds } from "@tripetto/slots/lib/kinds";
    export { TPipable } from "@tripetto/slots/lib/pipable";
    export {
        Transformations
    } from "@tripetto/slots/lib/types/text/transformations";
    /** Events */
    export {
        ISlotPropertyEvent,
        ISlotRenameEvent,
        ISlotRequiredEvent
    } from "@tripetto/slots/lib/events";
    /** Decorators */
    export { slot, property } from "@tripetto/slots/lib/decorators";
    /** Built-in types */
    export {
        BooleanSlot as Boolean
    } from "@tripetto/slots/lib/types/boolean/boolean";
    export { DateSlot as Date } from "@tripetto/slots/lib/types/date/date";
    export {
        NumberSlot as Number
    } from "@tripetto/slots/lib/types/number/number";
    export { Numeric } from "@tripetto/slots/lib/types/numeric/numeric";
    export {
        StringSlot as String
    } from "@tripetto/slots/lib/types/string/string";
    export { Text } from "@tripetto/slots/lib/types/text/text";
}

declare module "@tripetto/slots/lib/slots" {
    import { IDimensionChangeEvent, Matrix } from "@marblecore/matrix";
    import { ISlot } from "@tripetto/slots/lib/interface";
    import { ISlotProperties } from "@tripetto/slots/lib/properties";
    import { ISlotType } from "@tripetto/slots/lib/type";
    import { Kinds } from "@tripetto/slots/lib/kinds";
    import { Slot } from "@tripetto/slots/lib/slot";
    import { TSlotEvents } from "@tripetto/slots/lib/events";
    export class Slots extends Matrix<Slot, TSlotEvents> {
        /**
         * Registers a slot type.
         * @param pType Specifies the slot instance constructor.
         * @param sType Specifies the slot type.
         */
        static register<T extends Slot>(
            pType: ISlotType<T>,
            sType: string
        ): void;
        /**
         * Retrieves the slot constructor of the specified type.
         * @param sType Specifies the slot type.
         * @return Returns the slot constructor or `undefined` if the slot type is not
         * available.
         */
        static ofType<T extends Slot>(sType: string): ISlotType<T> | undefined;
        /** Constructs a new slots instance. */
        constructor();
        /** Retrieves if the slots are being deserialized. */
        readonly isDeserializing: boolean;
        /** Retrieves if multiple pipable slots are present. */
        readonly hasMultiplePipable: boolean;
        /**
         * Fired when the amount of slots is changed.
         * @param pEvent Specifies the event payload.
         * @event
         */
        protected onDimensionChange(pEvent: IDimensionChangeEvent<this>): void;
        /**
         * Creates a new meta slot.
         * @param pProperties Specifies the properties for the slot.
         * @return Returns a reference to the slot.
         */
        meta<T extends Slot>(pProperties: ISlotProperties<T>): T;
        /**
         * Creates a new static slot.
         * @param pProperties Specifies the properties for the slot.
         * @return Returns a reference to the slot.
         */
        static<T extends Slot>(pProperties: ISlotProperties<T>): T;
        /**
         * Creates a new dynamic slot.
         * @param pProperties Specifies the properties for the slot.
         * @return Returns a reference to the slot.
         */
        dynamic<T extends Slot>(pProperties: ISlotProperties<T>): T;
        /**
         * Creates a new feature slot.
         * @param pProperties Specifies the properties for the slot.
         * @return Returns a reference to the slot.
         */
        feature<T extends Slot>(pProperties: ISlotProperties<T>): T;
        /**
         * Selects a slot.
         * @param sReferenceOrId Reference or identifier of the slot.
         * @param sKind Specifies the optional kind of the slot.
         * @return Returns the slot or `undefined` if no slot is found.
         */
        select<T extends Slot>(
            sReferenceOrId: string,
            sKind?: Kinds
        ): T | undefined;
        /**
         * Selects a slot (or slot group) by the specified identifier.
         * @param sId Specifies the identifier to search for.
         * @param sParent Specifies the parent or slot group identifier.
         * @return Returns the slot or `undefined` if no slot was found.
         */
        selectByIdentifier<T extends Slot>(
            sId: string,
            sParent?: string
        ): T | undefined;
        /**
         * Deprecates a slot.
         * @param sReferenceOrId Reference or identifier of the slot.
         * @param sKind Specifies the optional kind of the slot.
         * @return Returns the slot or `undefined` if no slot is found.
         */
        deprecate<T extends Slot>(
            sReferenceOrId: string,
            sKind?: Kinds
        ): T | undefined;
        /**
         * Deletes a slot.
         * @param sReferenceOrId Reference or identifier of the slot.
         * @param sKind Specifies the optional kind of the slot.
         * @return Returns the slot or `undefined` if no slot is found.
         */
        delete<T extends Slot>(
            sReferenceOrId: string,
            sKind?: Kinds
        ): T | undefined;
        /**
         * Removes all the slots from the collection except the supplied slots.
         * @param sKind Specifies the kind of slots to remove (or `undefined` to remove all).
         * @param pExclude Specifies the slots to exclude. Those are kept in the slots collection.
         */
        clean(sKind?: Kinds, ...pExclude: (Slot | undefined)[]): void;
        /**
         * Sorts the slots.
         * @return Returns `true` if the order of the slots is changed.
         */
        sort(): boolean;
        /**
         * Serializes the slots to an array.
         * @return Returns the slots array.
         */
        serialize(): ISlot[];
        /**
         * Deserializes slots.
         * @param pSlots Specifies the slots array.
         * @return Returns a reference to the instance.
         */
        deserialize(pSlots: ISlot[]): this;
        /** Destroys the slots instance. */
        destroy(): void;
    }
}

declare module "@tripetto/slots/lib/slot" {
    import { Item } from "@marblecore/matrix";
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slots } from "@tripetto/slots/lib/slots";
    import { TSlotEvents } from "@tripetto/slots/lib/events";
    import { ISlot } from "@tripetto/slots/lib/interface";
    import { ISlotType } from "@tripetto/slots/lib/type";
    import { Kinds } from "@tripetto/slots/lib/kinds";
    import { TPipable } from "@tripetto/slots/lib/pipable";
    export abstract class Slot<
        T extends TSerializeTypes = TSerializeTypes
    > extends Item<TSlotEvents> {
        /** Contains the static type identifier. */
        static readonly type: string;
        /** Contains the label of the slot. */
        label?: string;
        /** Contains the name of the slot. */
        name?: string;
        /** Contains the placeholder of the slot. */
        placeholder?: string;
        /** Contains an alias for the slot. */
        alias?: string;
        /** Contains if this slot is required. */
        required?: boolean;
        /** Contains if this slot is pipable. */
        pipable?: TPipable;
        /** Contains the default slot data value. */
        default?: T;
        /** Contains if this slot is exportable. */
        exportable?: boolean;
        /**
         * Creates a new slot instance.
         * @param pType Reference to the type constructor.
         * @param sKind Specifies the kind.
         * @param sReference Specifies the reference for the slot.
         * @param nSequence Specifies the sequence number.
         */
        constructor(
            pType: ISlotType,
            sKind: Kinds,
            sReference: string,
            nSequence?: number
        );
        /** Retrieves a reference to the slots. */
        readonly slots: Slots | undefined;
        /** Retrieves the identifier of the slot. */
        readonly id: string;
        /** Retrieves the slot type identifier. */
        readonly type: string;
        /** Retrieves the slot kind. */
        readonly kind: Kinds;
        /** Sets the sequence number. */
        sequence: number | undefined;
        /** Retrieves the slot reference. */
        readonly reference: string;
        /**
         * Emits the `OnSlotProperty` event.
         * @param sProperty Specifies the changed property.
         * @event
         */
        propertyChange(sProperty: keyof this): void;
        /**
         * Serializes a slot.
         * @return Returns a reference to the serialized slot.
         */
        serialize(): ISlot<T>;
        /**
         * Deserializes to a slot instance.
         * @param pSlot Contains the serialized slot.
         * @return Returns a reference to the instance.
         */
        deserialize(pSlot: ISlot<T>): this;
        /**
         * Deprecates a slot.
         * @return Returns a reference to the instance.
         */
        deprecate(): this;
        /**
         * Deletes a slot.
         * @return Returns a reference to the instance.
         */
        delete(): this;
        /**
         * Retrieves the group identifier if the slot is in a group.
         * @param sParent Specifies the parent identifier.
         * @return Returns the group identifier or an empty string if the slot is
         * not in a group.
         */
        groupIdentifier(sParent: string): string;
        /**
         * Converts the supplied data to a valid value.
         * @param pData Specifies the data.
         * @return Returns the data value.
         */
        abstract toValue(pData: TSerializeTypes): T;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        abstract toString(pData: TSerializeTypes): string;
    }
}

declare module "@tripetto/slots/lib/interface" {
    /** Dependencies */
    import { ISerialize, TSerializeTypes } from "@marblecore/serializer";
    import { ISlot as ISlotProps } from "@tripetto/slots/lib/properties";
    export interface ISlot<T extends TSerializeTypes = TSerializeTypes>
        extends ISlotProps<T>,
            ISerialize {}
}

declare module "@tripetto/slots/lib/properties" {
    /** Dependencies */
    import { TSerializeTypes } from "@marblecore/serializer";
    import { ISlotType } from "@tripetto/slots/lib/type";
    import { Kinds } from "@tripetto/slots/lib/kinds";
    import { Slot } from "@tripetto/slots/lib/slot";
    import { TPipable } from "@tripetto/slots/lib/pipable";
    export interface ISlot<T extends TSerializeTypes = TSerializeTypes> {
        /** Contains the id for the serialized slot. */
        readonly id: string;
        /** Contains the slot type. */
        readonly type: string;
        /** Contains the slot kind. */
        readonly kind: Kinds;
        /** Contains the slot reference. */
        readonly reference: string;
        /** Contains the slot sequence number. */
        readonly sequence?: number;
        /** Contains the label for the slot. */
        readonly label?: string;
        /** Contains the slot name. */
        readonly name?: string;
        /** Contains the slot placeholder. */
        readonly placeholder?: string;
        /** Contains the slot alias. */
        readonly alias?: string;
        /** Contains if the slot is required. */
        readonly required?: boolean;
        /** Contains if the slot is pipable. */
        readonly pipable?: TPipable;
        /** Contains the default slot value. */
        readonly default?: T;
        /** Contains if this slot is exportable. */
        readonly exportable?: boolean;
    }
    export interface ISlotProperties<T extends Slot>
        extends Partial<
            Pick<
                ISlot,
                Exclude<keyof ISlot, "id" | "type" | "kind" | "reference">
            >
        > {
        /** Contains the slot type. */
        readonly type: ISlotType<T>;
        /** Contains the slot reference. */
        readonly reference: string;
        /** Contains the label for the slot. */
        readonly label: string;
    }
}

declare module "@tripetto/slots/lib/type" {
    /** Dependencies */
    import { Slot } from "@tripetto/slots/lib/slot";
    import { Kinds } from "@tripetto/slots/lib/kinds";
    export interface ISlotType<T extends Slot = Slot> {
        /** Contains the slot type identifier. */
        readonly type: string;
        /**
         * Creates a new slot instance.
         * @param pType Reference to the type constructor.
         * @param sKind Specifies the kind.
         * @param sReference Specifies the reference.
         * @param nSequence Specifies the sequence number.
         */
        new (
            pType: ISlotType,
            sKind: Kinds,
            sReference: string,
            nSequence?: number
        ): T;
    }
}

declare module "@tripetto/slots/lib/kinds" {
    /** Specifies the slot kinds. */
    export type Kinds = "meta" | "static" | "dynamic" | "feature";
    /**
     * Retrieves the static order index for the supplied kind.
     * @param sKind Specifies the kind.
     * @return Returns the order index.
     */
    export function kindOrderIndexFor(sKind: Kinds): number;
}

declare module "@tripetto/slots/lib/pipable" {
    export type TPipable =
        | boolean
        | {
              /** Group identifier. */
              group: string;
              /** Optional localized label for the group. */
              label?: string;
              /** Optional alias for the group. */
              alias?: string;
              /** Specifies the field used as the group data (defaults to `value`). */
              template?: "value" | "label" | "name";
          };
}

declare module "@tripetto/slots/lib/types/text/transformations" {
    export type Transformations =
        | "none"
        | "capitalize"
        | "capitalize-words"
        | "capitalize-sentences"
        | "uppercase"
        | "lowercase";
}

declare module "@tripetto/slots/lib/events" {
    /** Dependencies */
    import { IHookPayload, THooks } from "@marblecore/hookup";
    import { Slot } from "@tripetto/slots/lib/slot";
    export type TSlotEvents =
        | "OnSlotRename"
        | "OnSlotRequired"
        | "OnSlotProperty";
    export interface ISlotEvent<H extends THooks, T extends Slot>
        extends IHookPayload<H> {
        /** Reference to the slot. */
        readonly slot: T;
    }
    export interface ISlotRenameEvent<T extends Slot>
        extends ISlotEvent<"OnSlotRename", T> {}
    export interface ISlotRequiredEvent<T extends Slot>
        extends ISlotEvent<"OnSlotRequired", T> {}
    export interface ISlotPropertyEvent<T extends Slot>
        extends ISlotEvent<"OnSlotProperty", T> {
        /** Specifies the changed property. */
        readonly property: keyof T;
    }
}

declare module "@tripetto/slots/lib/decorators" {
    /** Dependencies */
    import { ISlotType } from "@tripetto/slots/lib/type";
    import { Slot } from "@tripetto/slots/lib/slot";
    /**
     * Defines a slot type.
     * @param sType Specifies the slot type name.
     */
    export function slot<T extends Slot>(
        sType: string
    ): (pType: ISlotType<T>) => void;
    /**
     * Marks a property as a slot property.
     * @param pTarget Reference to the slot.
     * @param sProperty Specifies the slot property.
     * @param pDescriptor Reference to the property descriptor.
     */
    export function property(
        pTarget: Slot,
        sProperty: string,
        pDescriptor?: PropertyDescriptor
    ): void;
}

declare module "@tripetto/slots/lib/types/boolean/boolean" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    export class BooleanSlot extends Slot<boolean> {
        /** Contains the string label for false values. */
        labelForFalse?: string;
        /** Contains the string label for true values. */
        labelForTrue?: string;
        /**
         * Converts the supplied data to a valid boolean value.
         * @param pData Specifies the data.
         * @return Returns the boolean value.
         */
        toValue(pData: TSerializeTypes): boolean;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        toString(pData: TSerializeTypes): string;
    }
}

declare module "@tripetto/slots/lib/types/date/date" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    export class DateSlot extends Slot<number> {
        /** Contains the minimum date. */
        minimum?: Date | number;
        /** Contains the maximum date. */
        maximum?: Date | number;
        /**
         * Converts the supplied data to a date value.
         * @param pData Specifies the data.
         * @return Returns the date value.
         */
        toValue(pData: TSerializeTypes | Date): number;
        /**
         * Converts the supplied data to a valid date object.
         * @param pData Specifies the data.
         * @return Returns the data as a date object.
         */
        toDate(pData: TSerializeTypes | Date): Date;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        toString(pData: TSerializeTypes | Date): string;
    }
}

declare module "@tripetto/slots/lib/types/number/number" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    export class NumberSlot extends Slot<number> {
        /**
         * Converts the supplied data to a valid number value.
         * @param pData Specifies the data.
         * @return Returns the number value.
         */
        toValue(pData: TSerializeTypes): number;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        toString(pData: TSerializeTypes): string;
    }
}

declare module "@tripetto/slots/lib/types/numeric/numeric" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    export class Numeric extends Slot<number> {
        /** Contains the minimal value. */
        minimum?: number;
        /** Contains the maximum value. */
        maximum?: number;
        /** Contains the thousands separtor sign. */
        separator?: string;
        /** Contains the decimal sign. */
        decimal?: string;
        /** Contains the minus sign. */
        minus?: string;
        /** Contains the prefix. */
        prefix?: string;
        /** Contains the suffix. */
        suffix?: string;
        /** Contains the number of digits. */
        digits?: number;
        /** Sets the numeric precision. */
        precision: number | undefined;
        /**
         * Converts the supplied data to a valid numeric value.
         * @param pData Specifies the data.
         * @return Returns the numeric value.
         */
        toValue(pData: TSerializeTypes): number;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        toString(pData: TSerializeTypes): string;
    }
}

declare module "@tripetto/slots/lib/types/string/string" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    export class StringSlot extends Slot<string> {
        /**
         * Converts the supplied data to a valid string value.
         * @param pData Specifies the data.
         * @return Returns the string value.
         */
        toValue(pData: TSerializeTypes): string;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        toString(pData: TSerializeTypes): string;
    }
}

declare module "@tripetto/slots/lib/types/text/text" {
    import { TSerializeTypes } from "@marblecore/serializer";
    import { Slot } from "@tripetto/slots/lib/slot";
    import { Transformations } from "@tripetto/slots/lib/types/text/transformations";
    export class Text extends Slot<string> {
        /** Sets the maximum text length. */
        maxLength: number | undefined;
        /** Specifies the transformation to apply. */
        transformation?: Transformations;
        /**
         * Converts the supplied data to a valid text value.
         * @param pData Specifies the data.
         * @return Returns the text value.
         */
        toValue(pData: TSerializeTypes): string;
        /**
         * Converts the supplied data to a valid string.
         * @param pData Specifies the data.
         * @return Returns the data as a string.
         */
        toString(pData: TSerializeTypes): string;
    }
}
